@@ -25,6 +25,32 @@ def get_arguments():
     from argparse import ArgumentParser
     parser = ArgumentParser(description='AWS Kommandos Script')
 
+    credentials = parser.add_argument_group('AWS Credentials')
+    credentials.add_argument('--access-key-id',
+                             dest='access_key_id',
+                             required=False,
+                             type=str,
+                             help="Specify the AWS access key ID to use. "
+                                  "If omitted, the script grabs the key ID from the AWS_ACCESS_KEY_ID env variable. "
+                                  "If that variable does not present, the script takes default credentials "
+                                  "located in the ~/.aws/credentials file.")
+    credentials.add_argument('--access-key-secret',
+                             dest='access_key_secret',
+                             required=False,
+                             type=str,
+                             help="Specify the AWS access key secret to use. "
+                                  "If omitted, the script grabs the key secret from the AWS_ACCESS_KEY_SECRET env variable. "
+                                  "If that variable does not present, the script takes default credentials "
+                                  "located in the ~/.aws/credentials file.")
+    credentials.add_argument('--region-name',
+                             dest='region_name',
+                             required=False,
+                             type=str,
+                             help="Specify the AWS region to use. "
+                                  "If omitted, the script grabs the key secret from the AWS_REGION env variable. "
+                                  "If that variable does not present, the script takes default credentials "
+                                  "located in the ~/.aws/config file.")
+
     # EC2 Starting Instances
     starting_commands = parser.add_argument_group('Starting Instances')
     starting_commands.add_argument("--start",
@@ -256,6 +282,16 @@ def get_arguments():
                                "Takes the value from the --key-pair-name parameter.")
 
     options = parser.parse_args()
+    if options.access_key_id and not options.access_key_secret:
+        parser.error(f"The --access-key-id argument must go together with --access-key-secret argument. "
+                     f"Use --help for more info.")
+    if not options.access_key_id and options.access_key_secret:
+        parser.error(f"The --access-key-secret argument must go together with --access-key-id argument. "
+                     f"Use --help for more info.")
+    if (options.access_key_id and options.access_key_secret) and not options.region_name:
+        parser.error("The region name must be supplied with the --region-name argument when credentials"
+                     " are passed via command line."
+                     "Use --help for more info.")
     if options.invoke_script_argument:
         if not options.invoke_script:
             parser.error(f"--invoke-script-argument has been passed "
@@ -320,7 +356,7 @@ def get_arguments():
     return options
 
 
-class kommandos:
+class FirewallRuleRequest:
     def __init__(self, rule_from_command_line):
         if ':' not in rule_from_command_line:
             raise Exception('Invalid format of the firewall rule. Use --help to see an example.')
@@ -347,10 +383,23 @@ class kommandos:
 
 
 class AwsManager:
-    def __init__(self):
-        self.ec2 = boto3.resource('ec2')
-        self.route53_client = boto3.client('route53')
-        self.ec2_client = boto3.client('ec2')
+    def __init__(self,
+                 aws_access_key_id: str = None,
+                 aws_access_key_secret: str = None,
+                 region_name: str = None):
+        try:
+            self.ec2 = boto3.resource('ec2', aws_access_key_id=aws_access_key_id,
+                                      aws_access_key_secret=aws_access_key_secret,
+                                      region_name=region_name)
+            self.route53_client = boto3.client('route53', aws_access_key_id=aws_access_key_id,
+                                               aws_access_key_secret=aws_access_key_secret,
+                                               region_name=region_name)
+            self.ec2_client = boto3.client('ec2', aws_access_key_id=aws_access_key_id,
+                                           aws_access_key_secret=aws_access_key_secret,
+                                           region_name=region_name)
+        except Exception as e:
+            print(f"Kommandos initialization failed: {e}")
+            exit(1)
 
     ## DNS
     ### GETTING ALL HOSTING ZONES
@@ -570,7 +619,7 @@ class AwsManager:
                 print(colored(f"{type(e)} - {e}", 'red'))
 
     ### CONFIGURING DA FIREWALL
-    def add_ingress_rule(self, firewall_rule_request: kommandos, security_group_id: str):
+    def add_ingress_rule(self, firewall_rule_request: FirewallRuleRequest, security_group_id: str):
         print(f"Authorizing ingress '{firewall_rule_request}' on '{security_group_id}'")
         if not firewall_rule_request.description:
             description = f"{firewall_rule_request.port}-{firewall_rule_request.protocol}-custom"
@@ -609,7 +658,7 @@ class AwsManager:
             else:
                 print(f"{type(e)} - {e}")
 
-    def add_egress_rule(self, firewall_rule_request: kommandos, security_group_id: str):
+    def add_egress_rule(self, firewall_rule_request: FirewallRuleRequest, security_group_id: str):
         print(f"Authorizing egress '{firewall_rule_request}' on '{security_group_id}'")
         if not firewall_rule_request.description:
             description = f"{firewall_rule_request.port}-{firewall_rule_request.protocol}-custom"
@@ -648,7 +697,7 @@ class AwsManager:
             else:
                 print(f"{type(e)} - {e}")
 
-    def delete_ingress_rule(self, firewall_rule_request: kommandos, security_group_id: str):
+    def delete_ingress_rule(self, firewall_rule_request: FirewallRuleRequest, security_group_id: str):
         print(f"Revoking ingress '{firewall_rule_request}' on '{security_group_id}'")
 
         # find the description if the one wasn't supplied
@@ -697,7 +746,7 @@ class AwsManager:
         except botocore.client.ClientError as e:
             print(f"{type(e)} - {e}")
 
-    def delete_egress_rule(self, firewall_rule_request: kommandos, security_group_id: str):
+    def delete_egress_rule(self, firewall_rule_request: FirewallRuleRequest, security_group_id: str):
         print(f"Revoking egress '{firewall_rule_request}' on '{security_group_id}'")
 
         # find the description if the one wasn't supplied
@@ -926,11 +975,11 @@ class AwsManager:
         if new_instance:
             print(colored('The instance has been created', 'green'))
             print('Waiting for the server boot...')
-            instance = aws_manager.get_instance(new_instance['InstanceId'])
+            instance = self.get_instance(new_instance['InstanceId'])
             instance.wait_until_running()
             print(f'The server is up and running at {colored(instance.public_ip_address, "green")}')
             print('Waiting until the SSH service is available...')
-            aws_manager.poll_ssh_status(instance.id)
+            self.poll_ssh_status(instance.id)
             identified_user_name = self.get_default_ami_user_name(instance.image_id)
             print("Use the following command for connecting to the instance: " +
                   colored(f"ssh {identified_user_name}@{instance.public_ip_address} -i {key_pair_name}.pem", 'green'))
@@ -988,9 +1037,27 @@ class AwsManager:
         if script_name.endswith('-kommandos-temp'):
             os.remove(script_name)
 
+
 def main():
     options = get_arguments()
 
+    # find credentials
+    if options.access_key_id and options.access_key_secret and options.region_name:
+        access_key_id = options.access_key_id
+        access_key_secret = options.access_key_secret
+        region_name = options.region_name
+    elif 'AWS_ACCESS_KEY_ID' in os.environ \
+            and 'AWS_ACCESS_KEY_SECRET' in os.environ \
+            and 'AWS_REGION' in os.environ:
+        access_key_id = os.environ['AWS_ACCESS_KEY_ID']
+        access_key_secret = os.environ['AWS_ACCESS_KEY_SECRET']
+        region_name = os.environ['AWS_REGION']
+    else:
+        # assume the AWS client's been already configured
+        access_key_id = None
+        access_key_secret = None
+        region_name = None
+
     key_pair_name = options.key_pair_name
     image_id = options.image_id
     security_group_id = options.security_group_id
@@ -998,7 +1065,9 @@ def main():
 
     domain_name = options.fqdn
 
-    aws_manager = AwsManager()
+    aws_manager = AwsManager(aws_access_key_id=access_key_id,
+                             aws_access_key_secret=access_key_secret,
+                             region_name=region_name)
 
     if options.get_ami:
         image = aws_manager.get_ami_image(image_id=image_id)
@@ -1040,19 +1109,19 @@ def main():
 
     if options.allow_inbound:
         for rule in options.allow_inbound:
-            aws_manager.add_ingress_rule(firewall_rule_request=kommandos(rule),
+            aws_manager.add_ingress_rule(firewall_rule_request=FirewallRuleRequest(rule),
                                          security_group_id=security_group_id)
     if options.allow_outbound:
         for rule in options.allow_outbound:
-            aws_manager.add_egress_rule(firewall_rule_request=kommandos(rule),
+            aws_manager.add_egress_rule(firewall_rule_request=FirewallRuleRequest(rule),
                                         security_group_id=security_group_id)
     if options.delete_inbound:
         for rule in options.delete_inbound:
-            aws_manager.delete_ingress_rule(firewall_rule_request=kommandos(rule),
+            aws_manager.delete_ingress_rule(firewall_rule_request=FirewallRuleRequest(rule),
                                             security_group_id=security_group_id)
     if options.delete_outbound:
         for rule in options.delete_outbound:
-            aws_manager.delete_egress_rule(firewall_rule_request=kommandos(rule),
+            aws_manager.delete_egress_rule(firewall_rule_request=FirewallRuleRequest(rule),
                                            security_group_id=security_group_id)
 
     if options.stats:
@@ -1117,5 +1186,6 @@ def main():
                                       file_name=options.invoke_script,
                                       parameters=options.invoke_script_argument)
 
+
 if __name__ == '__main__':
     main()
